import java.io.FileNotFoundException;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

/**
 * Read a graph into memory.
 * 
 * @author carlk
 */
public class Graph {
	public Graph() {
		_adj = new HashMap<String, List<String>>();
		_color = new HashMap<String, Integer>();
		_weights = new HashMap<Set<String>, Double>();
	}
	
	/**
	 * Add edge between nodes a and b, adding a and b as nodes if needed
	 * @param a - first node
	 * @param b - second node
	 */
	public void addEdge(String a, String b, double wt) {
		// if the edge isn't there
		if (!_adj.get(a).contains(b) &&!_adj.get(b).contains(a)) {
			
			// add the edge
			_adj.get(a).add(b);
			_adj.get(b).add(a);

			// set it's weight
			setWeight(a,b,wt);
		}
	}
	
	/**
	 * Create a node if it does not exist; nop if it already exists
	 * @param a - the node to create
	 */
	public void addNode(String a) {
		if (!_adj.containsKey(a)) {
			_adj.put(a, new LinkedList<String>());
		}
	}
	
	/**
	 * Format:
	 * 		a 1 b c d e
	 * 		d 2 c e d f
	 * @param filename
	 * @throws FileNotFoundException
	 * @throws Exception
	 */
	public void readGraph(String [][] entries, int numEntries)  {

		for (int i = 0; i < numEntries; i++) {
			String node1 = entries[i][0];
			
			addNode(node1);
			_color.put(node1, Integer.parseInt(entries[i][1]));
			
			if(entries[i][2] != null) {
				float wt = Float.parseFloat(entries[i][2]);
			
				int j = 3;
				while (entries[i][j] != null) {
					String node2 = entries[i][j];
					addNode(node2);
					addEdge(node1, node2, wt);
					j++;
				}
			}
		}
		
		// default colors 
		for(String a : _adj.keySet()) {
			if(!_color.containsKey(a)) _color.put(a, 0);
		}
	}
	
	/**
	 * Return number of vertices
	 * 
	 * @return
	 */
	public int numVertices() {
		return _adj.size();
	}
	
	/**
	 * Return # of edges
	 * @return number of edges
	 */
	public int numEdges() {
		int total = 0;
		for (List<String> a : _adj.values()) {
			total += a.size();
		}
		return total / 2;	
	}
	
	/**
	 * Return the maximum color used.
	 * @return the max color
	 */
	public int maxColor() {
		int m = 0;
		for(int c : _color.values()) {
			m = Math.max(c,m);
		}
		return m;
	}
	
	/**
	 * Return the color.
	 * @param u
	 * @return
	 */
	public int color(String u) {
		return _color.get(u);
	}
	
	
	/**
	 * Remove the given node from the graph.
	 * 
	 * @param u - the node to remove
	 */
	public void removeNode(String u) {
		if(_adj.containsKey(u)) {
			for(String v : getNeighbors(u)) {
				_adj.get(v).remove(u);
			}
			_adj.remove(u);
			_color.remove(u);
		}
	}
	
	/**
	 * Return the BFS levels starting from u.
	 * 
	 * @param u - the node to start from
	 * @return a mapping between level # and sets of nodes
	 */
	public Map<Integer, Set<String>> getBFSLevels(String u) {
		Map<Integer, Set<String>> levels = new HashMap<Integer, Set<String>>();
		
		// create the base case
		Set<String> lev = new TreeSet<String>();
		lev.add(u);
		levels.put(0, lev);
		int i = 0;
		do {
			lev = new TreeSet<String>();
			for (String v : levels.get(i)) {
				lev.addAll(getNeighbors(v));
			}
			
			// remove those nodes already visited in the BFS
			lev.removeAll(levels.get(i));
			if(i > 0) lev.removeAll(levels.get(i-1));
			
			// save the BFS
			if(!lev.isEmpty()) levels.put(i+1, lev);
			i++;

		} while(!lev.isEmpty());
		
		//System.out.println("LEV=" + levels);
		return levels;
	}
	
	/**
	 * Return the degree of node u
	 * @param u - a graph node
	 * @return its degree
	 */
	public int degree(String u) {
		return _adj.get(u).size();
	}
	
	/**
	 * Return the set of vertices of the graph
	 * @return the set of vertices
	 */
	public Collection<String> vertices() {
		return _adj.keySet();
	}
	
	/**
	 * Return the neighbors of node u
	 * @param u - a graph node
	 * @return the set of neighbors
	 */
	public Collection<String> getNeighbors(String u) {
		return _adj.get(u);
	}
	
	/**
	 * Make a copy of this graph.
	 */
	public Graph clone() {
		Graph g = new Graph();
		g._adj = new HashMap<String, List<String>>(_adj);
		for(String v : _adj.keySet()) {
			g._adj.put(v, new LinkedList<String>(_adj.get(v)));
		}
		g._color = new HashMap<String, Integer>(_color);
		return g;
	}
	
	/**
	 * Returns true if the graph contains node u
	 * @param u - node name
	 * @return true if u is a node of this graph
	 */
	public boolean contains(String u) {
		return _adj.containsKey(u);
	}
	
	/**
	 * Get the weight of an edge
	 * @param a - the first node in the edge
	 * @param b - the second node in the edge
	 * @return the weight of edge {a,b}
	 */
	public double weight(String a, String b) {
		Set<String> ab = new TreeSet<String>();
		ab.add(a);
		ab.add(b);
		if(_weights.containsKey(ab)) {
			return _weights.get(ab);
		}
		return 0.0;
	}
	
	public double totalWeight() {
		double d = 0.0;
		for(double wt : _weights.values()) d += wt;
		return d;
	}
	
	/**
	 * Set the weight of an edge.
	 * @param a - first node in edge
	 * @param b - second node in edge
	 * @param wt - the new weight
	 */
	public void setWeight(String a, String b, double wt) {
		Set<String> ab = new TreeSet<String>();
		ab.add(a);
		ab.add(b);
		_weights.put(ab, wt);
	}
	
	/** the adjaceny lists */
	private Map<String, List<String>> _adj;
	
	/** the node colors */
	private Map<String, Integer> _color;
	
	/** edge weights */
	private Map<Set<String>, Double> _weights;
	
	/**
	 * P2 - Read in a graph and write the # of edges and nodes.
	 * @param args
	 */

}
