import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Locale;
import java.util.Map;
import java.util.List;
import java.util.LinkedList;
import java.util.HashMap;
import java.util.Scanner;
import java.util.Collection;
import java.util.Set;
import java.util.TreeSet;
import java.io.File;
import java.io.FileNotFoundException;
import java.lang.Exception;
import java.lang.Integer;

/**
 * Read a graph into memory.
 * 
 * @author carlk
 */
public class Graph {
	
	static final int MAXNODE = 1000;

	public Graph() {
		_adj = new HashMap<String, List<String>>();
		_color = new HashMap<String, Integer>();
		_weights = new HashMap<Set<String>, Double>();
	}
	
	/**
	 * Add edge between nodes a and b, adding a and b as nodes if needed
	 * @param a - first node
	 * @param b - second node
	 */
	public void addEdge(String a, String b, double wt) {
		// if the edge isn't there
		if (!_adj.get(a).contains(b) &&!_adj.get(b).contains(a)) {
			
			// add the edge
			_adj.get(a).add(b);
			_adj.get(b).add(a);

			// set it's weight
			setWeight(a,b,wt);
		}
	}
	
	/**
	 * Create a node if it does not exist; nop if it already exists
	 * @param a - the node to create
	 */
	public void addNode(String a) {
		if (!_adj.containsKey(a)) {
			_adj.put(a, new LinkedList<String>());
		}
	}
	
	/**
	 * Format:
	 * 		a 1 b c d e
	 * 		d 2 c e d f
	 * @param filename
	 * @throws FileNotFoundException
	 * @throws Exception
	 */
	
	public void readGraph(String [][] entries, int numEntries)  {

		for (int i = 0; i < numEntries; i++) {
			String node1 = entries[i][0];

			addNode(node1);
			_color.put(node1, 1);

			int j = 1;
			while (entries[i][j] != null) {
				String node2 = entries[i][j];
				addNode(node2);
				addEdge(node1, node2, 1.0);
				j++;
			}
		}
		
		// default colors 
		for(String a : _adj.keySet()) {
			if(!_color.containsKey(a)) _color.put(a, 0);
		}
	}
	
	/**
	 * Return number of vertices
	 * 
	 * @return
	 */
	public int numVertices() {
		return _adj.size();
	}
	
	/**
	 * Return # of edges
	 * @return number of edges
	 */
	public int numEdges() {
		int total = 0;
		for (List<String> a : _adj.values()) {
			total += a.size();
		}
		return total / 2;	
	}
	
	/**
	 * Return the maximum color used.
	 * @return the max color
	 */
	public int maxColor() {
		int m = 0;
		for(int c : _color.values()) {
			m = Math.max(c,m);
		}
		return m;
	}
	
	/**
	 * Return the color.
	 * @param u
	 * @return
	 */
	public int color(String u) {
		return _color.get(u);
	}
	
	
	/**
	 * Remove the given node from the graph.
	 * 
	 * @param u - the node to remove
	 */
	public void removeNode(String u) {
		if(_adj.containsKey(u)) {
			for(String v : getNeighbors(u)) {
				_adj.get(v).remove(u);
			}
			_adj.remove(u);
			_color.remove(u);
		}
	}
	
	/**
	 * Return the BFS levels starting from u.
	 * 
	 * @param u - the node to start from
	 * @return a mapping between level # and sets of nodes
	 */
	public Map<Integer, Set<String>> getBFSLevels(String u) {
		Map<Integer, Set<String>> levels = new HashMap<Integer, Set<String>>();
		
		// create the base case
		Set<String> lev = new TreeSet<String>();
		lev.add(u);
		levels.put(0, lev);
		int i = 0;
		do {
			lev = new TreeSet<String>();
			for (String v : levels.get(i)) {
				lev.addAll(getNeighbors(v));
			}
			
			// remove those nodes already visited in the BFS
			lev.removeAll(levels.get(i));
			if(i > 0) lev.removeAll(levels.get(i-1));
			
			// save the BFS
			if(!lev.isEmpty()) levels.put(i+1, lev);
			i++;

		} while(!lev.isEmpty());
		
		//System.out.println("LEV=" + levels);
		return levels;
	}
	
	/**
	 * Return the degree of node u
	 * @param u - a graph node
	 * @return its degree
	 */
	public int degree(String u) {
		return _adj.get(u).size();
	}
	
	/**
	 * Return the set of vertices of the graph
	 * @return the set of vertices
	 */
	public Collection<String> vertices() {
		return _adj.keySet();
	}
	
	/**
	 * Return the neighbors of node u
	 * @param u - a graph node
	 * @return the set of neighbors
	 */
	public Collection<String> getNeighbors(String u) {
		return _adj.get(u);
	}
	
	/**
	 * Make a copy of this graph.
	 */
	public Graph clone() {
		Graph g = new Graph();
		g._adj = new HashMap<String, List<String>>(_adj);
		for(String v : _adj.keySet()) {
			g._adj.put(v, new LinkedList<String>(_adj.get(v)));
		}
		g._color = new HashMap<String, Integer>(_color);
		return g;
	}
	
	/**
	 * Returns true if the graph contains node u
	 * @param u - node name
	 * @return true if u is a node of this graph
	 */
	public boolean contains(String u) {
		return _adj.containsKey(u);
	}
	
	/**
	 * Get the weight of an edge
	 * @param a - the first node in the edge
	 * @param b - the second node in the edge
	 * @return the weight of edge {a,b}
	 */
	public double weight(String a, String b) {
		Set<String> ab = new TreeSet<String>();
		ab.add(a);
		ab.add(b);
		if(_weights.containsKey(ab)) {
			return _weights.get(ab);
		}
		return 0.0;
	}
	
	public double totalWeight() {
		double d = 0.0;
		for(double wt : _weights.values()) d += wt;
		return d;
	}
	
	/**
	 * Set the weight of an edge.
	 * @param a - first node in edge
	 * @param b - second node in edge
	 * @param wt - the new weight
	 */
	public void setWeight(String a, String b, double wt) {
		Set<String> ab = new TreeSet<String>();
		ab.add(a);
		ab.add(b);
		_weights.put(ab, wt);
	}
	
	/** the adjaceny lists */
	private Map<String, List<String>> _adj;
	
	/** the node colors */
	private Map<String, Integer> _color;
	
	/** edge weights */
	private Map<Set<String>, Double> _weights;
	

	static public void printEntries(String [][] entries, int numEntries) {
		for (int i = 0; i < numEntries; i++) {
			String node1 = entries[i][0];
			
			System.out.print("Nodes " + node1 + "  neighbors =");
			
			int j = 1;
			while (entries[i][j] != null) {
				String node2 = entries[i][j];
				System.out.print(" " + node2);
				j++;
			}

			System.out.println();
		}
	}
	
	static public String countNodesEdges(String [][] entries, int numEntries) {
		
		Graph g = new Graph();
		g.readGraph(entries, numEntries);
		int numNodes = g.numVertices();
		int numEdges = g.numEdges();
		
		String result = "NODES " + numNodes + " EDGES " + numEdges;
		return result;
	}
	
	static public void main(String [] args) {
		DecimalFormat fm1 = new DecimalFormat("######",
				new DecimalFormatSymbols(Locale.US));

		// read the graph
		Scanner inp = new Scanner(System.in);

		try {
			String line = inp.nextLine();
			if (!line.equals("GRAPH BEGIN"))
				throw new Exception();
			while (inp.hasNext()) {
				if (!inp.hasNext()) throw new Exception();

				// found new graph
				String [][] entries = new String [MAXNODE][];
				int numEntries = 0;

				while (inp.hasNext()) {
					String ln = inp.nextLine();
					if (ln.equals("GRAPH END")) 	//* end of graph specification
						break;

					String [] entry = new String[MAXNODE];
					int idx = 0;
					
					// read in entry (on single line)
					Scanner sc = new Scanner(ln);
					if(!sc.hasNext()) throw new Exception();
					entry[idx++] = sc.next();		// node

					while (sc.hasNext()) {
						entry[idx++] = sc.next();	// neighbors
					}

					entry[idx] = null;
					entries[numEntries++] = entry;	// save entry 
				}

				// printEntries(entries, numEntries);
				String counts = countNodesEdges(entries, numEntries);
				System.out.println(counts);

				// process lines after graph, if any
				while (inp.hasNext()) {
					line = inp.nextLine();
					if (line.equals("GRAPH BEGIN")) {
						break;
					}
				}
			}
		} catch(Exception e) {
			System.out.println("BAD INPUT FORMAT");
		}
	}
	
}
